@@ -7,7 +7,7 @@ use v5.20;
 use warnings;
 use experimental 'signatures', 'postderef';
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use parent 'Form::Tiny::Plugin';
 
@@ -54,7 +54,7 @@ CXC::Form::Tiny::Plugin::OptArgs2 - A Plugin to interface Form::Tiny with OptArg
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -68,7 +68,7 @@ version 0.02
      form_field 'file' => ( type => NonEmptyStr, default => sub { 'file.ext' } );
  
      # the 'option' keyword immediately follows the field definition
-     option (
+     option(
          isa      => 'Str',
          comment  => 'Query in a file',
          isa_name => 'ADQL in a file',
@@ -78,7 +78,7 @@ version 0.02
      # specify their order on the command line
  
      form_field 'arg2' => ( type => ArrayRef, );
-     argument (
+     argument(
          isa     => 'ArrayRef',
          comment => 'every thing else',
          greedy  => 1,
@@ -86,7 +86,7 @@ version 0.02
      );
  
      form_field 'arg1' => ( type => NonEmptyStr, );
-     argument (
+     argument(
          isa     => 'Str',
          comment => 'first argument',
          order   => 1,
@@ -153,15 +153,11 @@ their meaning):
         default      => Optional [Value|CodeRef],
         required     => Optional [Bool],
         hidden       => Optional [Bool],
-        isa          => Enum[ qw( ArrayRef Flag Bool Counter HashRef Int Num Str ) ],
+        isa          => Optional [Enum[ qw( ArrayRef Flag Bool Counter HashRef Int Num Str ) ] ],
         isa_name     => Optional [NonEmptySimpleStr],
         show_default => Optional [Bool],
         trigger      => Optional [CodeRef],
 
-C<name> will be taken from the immediately preceding C<form_field>
-specification if not provided.  The form field and option names need not
-be the same.
-
 Unlike in L<OptArgs2>, a leading prefix of C<--> in the
 value of the C<isa> attribute is optional.
 
@@ -176,16 +172,36 @@ their meaning, except for C<order>):
         default      => Optional [Value|CodeRef],
         greedy       => Optional [Bool],
         fallthru     => Optional [Bool],
-        isa          => Enum [qw( ArrayRef HashRef Int Num Str SubCmd)],
+        isa          => Optional [Enum [qw( ArrayRef HashRef Int Num Str SubCmd)]],
         isa_name     => Optional [NonEmptySimpleStr],
         required     => Optional [Bool],
         show_default => Optional [Bool],
         order        => Int,
 
+The C<order> attribute specifies the relative order the argument
+should appear on the command line.  This allows the form fields to be
+specified in an arbitrary order.
+
+=head3 Automatic determination of OptArgs2 attributes
+
+=head4 C<name>
+
 C<name> will be taken from the immediately preceding C<form_field>
-specification if not provided.  The C<order> attribute specifies the
-relative order the argument should appear on the command line.  This
-allows the form fields to be specified in an arbitrary order.
+specification if not provided.  The form field and option names need
+not be the same.
+
+=head4 C<isa>
+
+The C<isa> option can be omitted in the list of option and argument
+attributes if the L<OptArgs2> type can be deduced from the
+L<Form::Tiny> type.  The types which require explicit specification
+are the L<OptArgs2> C<Flag>, C<Counter>, and C<SubCmd> types.
+
+=head4 C<required>
+
+L<OptArgs2> has a simpler concept of a required element than does
+L<Form::Tiny>.  If the field's required attribute is false, the option
+or argument's value is set to false, otherwise it is set to true.
 
 =head2 Interfacing with L<OptArgs2>
 
