@@ -6,7 +6,7 @@ use v5.20;
 
 use warnings;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use Hash::Fold 'unfold';
 use List::Util   qw( first );
@@ -21,8 +21,6 @@ use Moo::Role;
 
 use experimental 'signatures', 'postderef', 'lexical_subs';
 
-use constant { OPTION => 'option', ARGUMENT => 'argument' };
-
 use namespace::clean;
 
 my sub croak {
@@ -115,7 +113,7 @@ sub _create_options (
             my @paths = ( [ $path->@*, $field ] );
 
             if ( $is_subform
-                and defined( my $name = ( ( ( $def->data // {} )->{optargs} // {} )->{ +OPTION } // {} )->{name} ) )
+                and defined( my $name = ( ( $def->data // {} )->{optargs} // {} )->{name} ) )
             {
                 my @comp  = split( /[.]/, $def->name );
                 my @fixup = $opt_path->@*;
@@ -205,19 +203,45 @@ sub _create_options (
         ) ];
 }
 
-my sub optarg ( $opt_or_arg, $field, $data ) {
+my sub optarg ( $field, $spec ) {
     $field->set_data( {} ) unless $field->has_data;
     my $optargs = ( $field->data->{optargs} //= {} );
     croak( sprintf( 'duplicate definition for field %s', $field->name ) )
       if defined $optargs->{spec};
-    $optargs->{spec} = $data;
+
+    $spec->{required} //= !!$field->required;
+    $optargs->{spec} = $spec;
     return;
 }
 
 use constant OptionTypeEnums => qw( ArrayRef Flag Bool Counter HashRef Int Num Str );
-use constant OptionType => Enum( [ map { q{--} . $_ } OptionTypeEnums ] )
+use constant OptionTypeMap   => { map { $_ => "--$_" } OptionTypeEnums };
+
+use constant OptionType => Enum( [ values OptionTypeMap->%* ] )
   ->plus_coercions( NonEmptySimpleStr, sub { /^--/ ? $_ : "--$_" } );
 
+use constant ArgumentTypeEnums => qw( ArrayRef HashRef Int Num Str SubCmd );
+use constant ArgumentType      => Enum [ArgumentTypeEnums];
+use constant ArgumentTypeMap   => { map { $_ => $_ } ArgumentTypeEnums() };
+
+sub _resolve_type ( $field, $type_set ) {
+
+    # dynamic fields don't have types
+    return undef
+      unless defined $field
+      && $field->isa( 'Form::Tiny::FieldDefinition' )
+      && $field->has_type;
+
+    my $type = $field->type;
+
+    while ( defined $type ) {
+        return $type_set->{ $type->name } if exists $type_set->{ $type->name };
+        $type = $type->parent;
+    }
+
+    return undef;
+}
+
 signature_for _dsl_add_option => (
     method => 1,
     head   => 1,     # field context
@@ -229,16 +253,19 @@ signature_for _dsl_add_option => (
         default      => Optional [ Value | CodeRef ],
         required     => Optional [Bool],
         hidden       => Optional [Bool],
-        isa          => OptionType,
+        isa          => Optional [OptionType],
         isa_name     => Optional [NonEmptySimpleStr],
         show_default => Optional [Bool],
         trigger      => Optional [CodeRef],
     ],
 );
 sub _dsl_add_option ( $self, $context, $data ) {
-    _croak( q{The 'option' directive must be used after a field definition} )
+    croak( q{The 'option' directive must be used after a field definition} )
       if !defined( $context );
-    optarg( OPTION, $context, $data );
+    my %data = $data->%*;
+    $data{isa} //= _resolve_type( $context, OptionTypeMap )
+      // croak( sprintf( q{'isa' attribute not specified or resolved for %s}, $context->name ) );
+    optarg( $context, \%data );
 }
 
 signature_for _dsl_add_argument => (
@@ -251,7 +278,7 @@ signature_for _dsl_add_argument => (
         default      => Optional [ Value | CodeRef ],
         greedy       => Optional [Bool],
         fallthru     => Optional [Bool],
-        isa          => Enum [qw( ArrayRef HashRef Int Num Str SubCmd)],
+        isa          => Optional [ArgumentType],
         isa_name     => Optional [NonEmptySimpleStr],
         required     => Optional [Bool],
         show_default => Optional [Bool],
@@ -259,9 +286,12 @@ signature_for _dsl_add_argument => (
     ],
 );
 sub _dsl_add_argument ( $self, $context, $data ) {
-    _croak( q{The 'argument' directive must be used after a field definition} )
+    croak( q{The 'argument' directive must be used after a field definition} )
       if !defined( $context );
-    optarg( ARGUMENT, $context, $data );
+    my %data = $data->%*;
+    $data{isa} //= _resolve_type( $context, ArgumentTypeMap )
+      // croak( sprintf( q{'isa' attribute not specified or resolved for %s}, $context->name ) );
+    optarg( $context, \%data );
 }
 
 #
@@ -288,7 +318,7 @@ CXC::Form::Tiny::Plugin::OptArgs2::Meta - Form metaclass role for OptArgs2
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 DESCRIPTION
 
